import unittest
from smufolib.request import Request, URLWarning

# pylint: disable=C0115, C0116, C0103


class RequestTest(unittest.TestCase):

    def test_json(self):
        pass

    def test_raw(self):
        pass

    def test_path(self):
        pass

    def test_fallback(self):
        pass

    def test_mode(self):
        pass

    def test_encoding(self):
        pass


if __name__ == '__main__':
    unittest.main()
