import unittest
from smufolib.stdUtils import (
    flatten,
    addTuples,
    isFloat,
    validateClassAttr
)

# pylint: disable=C0115, C0116, C0103


class StdUtils(unittest.TestCase):

    def test_flatten(self):
        pass

    def test_addTuples(self):
        pass

    def test_isFloat(self):
        pass

    def test_validateClassAttr(self):
        pass


if __name__ == '__main__':
    unittest.main()
