import unittest
from smufolib.cli import commonParser

# pylint: disable=C0115, C0116, C0103


class CLI(unittest.TestCase):
    @classmethod
    def SetUpClass(cls):
        pass

    @classmethod
    def TearDownClass(cls):
        pass

    def test_commonParser(self):
        pass


if __name__ == '__main__':
    unittest.main()
