import unittest
from smufolib import Smufl

# pylint: disable=C0115, C0116, C0103


class EngravingDefaults(unittest.TestCase):
    def test_clear:
        pass

    def test_items:
        pass

    def test_keys:
        pass

    def test_update:
        pass

    def test_values:
        pass

    def test_smufl:
        pass

    def test_font:
        pass

    def test_glyph:
        pass

    def test_layer:
        pass

    def test_arrowShaftThickness:
        pass

    def test_barlineSeparation:
        pass

    def test_beamSpacing:
        pass

    def test_beamThickness:
        pass

    def test_bracketThickness:
        pass

    def test_dashedBarlineDashLength:
        pass

    def test_dashedBarlineGapLength:
        pass

    def test_dashedBarlineThickness:
        pass
        def test_hairpinThickness:
        pass

    def test_hBarThickness:
        pass

    def test_legerLineExtension:
        pass

    def test_legerLineThickness:
        pass

    def test_lyricLineThickness:
        pass

    def test_octaveLineThickness:
        pass

    def test_pedalLineThickness:
        pass

    def test_repeatBarlineDotSeparation:
        pass

    def test_repeatEndingLineThickness:
        pass

    def test_slurEndpointThickness:
        pass

    def test_slurMidpointThickness:
        pass

    def test_staffLineThickness:
        pass

    def test_stemThickness:
        pass

    def test_subBracketThickness:
        pass

    def test_textFontFamily:
        pass

    def test_textEnclosureThickness:
        pass

    def test_thickBarlineThickness:
        pass

    def test_thinBarlineThickness:
        pass

    def test_thinThickBarlineSeparation:
        pass

    def test_tieEndpointThickness:
        pass

    def test_tieMidpointThickness:
        pass

    def test_tieMidpointThickness:
        pass

    def test_tupletBracketThickness:
        pass

    def test_round:
        pass

    def test_spaces:
        pass


if __name__ == '__main__':
    unittest.main()
