import unittest
from smufolib import Smufl

# pylint: disable=C0115, C0116, C0103


class Smufl(unittest.TestCase):
    def test_font:
        pass

    def test_glyph:
        pass

    def test_layer:
        pass

    def test_designSize:
        pass

    def test_engravingDefaults:
        pass

    def test_sizeRange:
        pass

    def test_alternates:
        pass

    def test_anchors:
        pass

    def test_bBox:
        pass

    def test_codepoint:
        pass

    def test_componentGlyphs:
        pass

    def test_range:
        pass

    def test_advanceWidth:
        pass

    def test_version:
        pass

    def test_classes:
        pass

    def test_description:
        pass

    def test_name:
        pass

    def test_isLigature:
        pass

    def test_isMember:
        pass

    def test_isOptional:
        pass

    def test_isRecommended:
        pass

    def test_isSalt:
        pass

    def test_isSet:
        pass

    def test_round:
        pass

    def test_toSpaces:
        pass

    def test_toUnits:
        pass

    def test_spaces:
        pass

    def test_base:
        pass

    def test_findGlyph:
        pass

    def test_suffix:
        pass


if __name__ == '__main__':
    unittest.main()
