import unittest
from smufolib import Smufl

# pylint: disable=C0115, C0116, C0103


class Range(unittest.TestCase):
    def test_smufl:
        pass

    def test_glyph:
        pass

    def test_font:
        pass

    def test_layer:
        pass

    def test_name:
        pass

    def test_description:
        pass

    def test_glyphs:
        pass

    def test_start:
        pass

    def test_end:
        pass


if __name__ == '__main__':
    unittest.main()
